package org.apache.bcel.classfile;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.CharArrayReader;
import java.io.CharArrayWriter;
import java.io.FilterReader;
import java.io.FilterWriter;
import java.io.IOException;
import java.io.PrintStream;
import java.io.PrintWriter;
import java.io.Reader;
import java.io.Writer;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Locale;
import java.util.zip.GZIPInputStream;
import java.util.zip.GZIPOutputStream;
import org.apache.bcel.Const;
import org.apache.bcel.util.ByteSequence;
import org.apache.commons.lang3.ArrayUtils;

/* loaded from: org_apache_bcel_classfile.jar:org/apache/bcel/classfile/Utility.class */
public abstract class Utility {
    private static boolean wide;
    private static final char ESCAPE_CHAR = '$';
    private static final ThreadLocal<Integer> CONSUMER_CHARS = ThreadLocal.withInitial(() -> {
        return 0;
    });
    private static final int FREE_CHARS = 48;
    private static final int[] CHAR_MAP = new int[FREE_CHARS];
    private static final int[] MAP_CHAR = new int[256];

    /* loaded from: org_apache_bcel_classfile.jar:org/apache/bcel/classfile/Utility$JavaReader.class */
    private static class JavaReader extends FilterReader {
        public JavaReader(Reader in) {
            super(in);
        }

        @Override // java.io.FilterReader, java.io.Reader
        public int read() throws IOException {
            int b = this.in.read();
            if (b != Utility.ESCAPE_CHAR) {
                return b;
            }
            int i = this.in.read();
            if (i < 0) {
                return -1;
            }
            if ((i < Utility.FREE_CHARS || i > 57) && (i < 97 || i > 102)) {
                return Utility.MAP_CHAR[i];
            }
            int j = this.in.read();
            if (j < 0) {
                return -1;
            }
            char[] tmp = {(char) i, (char) j};
            return Integer.parseInt(new String(tmp), 16);
        }

        @Override // java.io.FilterReader, java.io.Reader
        public int read(char[] cbuf, int off, int len) throws IOException {
            for (int i = 0; i < len; i++) {
                cbuf[off + i] = (char) read();
            }
            return len;
        }
    }

    /* loaded from: org_apache_bcel_classfile.jar:org/apache/bcel/classfile/Utility$JavaWriter.class */
    private static class JavaWriter extends FilterWriter {
        public JavaWriter(Writer out) {
            super(out);
        }

        @Override // java.io.FilterWriter, java.io.Writer
        public void write(char[] cbuf, int off, int len) throws IOException {
            for (int i = 0; i < len; i++) {
                write(cbuf[off + i]);
            }
        }

        @Override // java.io.FilterWriter, java.io.Writer
        public void write(int b) throws IOException {
            if (Utility.isJavaIdentifierPart((char) b) && b != Utility.ESCAPE_CHAR) {
                this.out.write(b);
                return;
            }
            this.out.write(Utility.ESCAPE_CHAR);
            if (b >= 0 && b < Utility.FREE_CHARS) {
                this.out.write(Utility.CHAR_MAP[b]);
                return;
            }
            char[] tmp = Integer.toHexString(b).toCharArray();
            if (tmp.length == 1) {
                this.out.write(Utility.FREE_CHARS);
                this.out.write(tmp[0]);
            } else {
                this.out.write(tmp[0]);
                this.out.write(tmp[1]);
            }
        }

        @Override // java.io.FilterWriter, java.io.Writer
        public void write(String str, int off, int len) throws IOException {
            write(str.toCharArray(), off, len);
        }
    }

    static {
        int j = 0;
        for (int i = 65; i <= 90; i++) {
            CHAR_MAP[j] = i;
            MAP_CHAR[i] = j;
            j++;
        }
        for (int i2 = 103; i2 <= 122; i2++) {
            CHAR_MAP[j] = i2;
            MAP_CHAR[i2] = j;
            j++;
        }
        CHAR_MAP[j] = ESCAPE_CHAR;
        MAP_CHAR[ESCAPE_CHAR] = j;
        int j2 = j + 1;
        CHAR_MAP[j2] = 95;
        MAP_CHAR[95] = j2;
    }

    public static String accessToString(int accessFlags) {
        return accessToString(accessFlags, false);
    }

    public static String accessToString(int accessFlags, boolean forClass) {
        StringBuilder buf = new StringBuilder();
        int p = 0;
        int i = 0;
        while (p < 32768) {
            p = pow2(i);
            if ((accessFlags & p) != 0 && (!forClass || (p != 32 && p != 512))) {
                buf.append(Const.getAccessName(i)).append(" ");
            }
            i++;
        }
        return buf.toString().trim();
    }

    private static short byteToShort(byte b) {
        return b < 0 ? (short) (256 + b) : b;
    }

    public static String classOrInterface(int accessFlags) {
        return (accessFlags & 512) != 0 ? "interface" : "class";
    }

    public static int clearBit(int flag, int i) {
        int bit = pow2(i);
        return (flag & bit) == 0 ? flag : flag ^ bit;
    }

    public static String codeToString(byte[] code, ConstantPool constantPool, int index, int length) {
        return codeToString(code, constantPool, index, length, true);
    }

    public static String codeToString(byte[] code, ConstantPool constantPool, int index, int length, boolean verbose) {
        StringBuilder buf = new StringBuilder(code.length * 20);
        try {
            ByteSequence stream = new ByteSequence(code);
            Throwable th = null;
            for (int i = 0; i < index; i++) {
                try {
                    try {
                        codeToString(stream, constantPool, verbose);
                    } finally {
                    }
                } finally {
                }
            }
            int i2 = 0;
            while (stream.available() > 0) {
                if (length < 0 || i2 < length) {
                    String indices = fillup(stream.getIndex() + ":", 6, true, ' ');
                    buf.append(indices).append(codeToString(stream, constantPool, verbose)).append('\n');
                }
                i2++;
            }
            if (stream != null) {
                if (0 != 0) {
                    try {
                        stream.close();
                    } catch (Throwable th2) {
                        th.addSuppressed(th2);
                    }
                } else {
                    stream.close();
                }
            }
            return buf.toString();
        } catch (IOException e) {
            throw new ClassFormatException("Byte code error: " + buf.toString(), e);
        }
    }

    public static String codeToString(ByteSequence bytes, ConstantPool constantPool) throws IOException {
        return codeToString(bytes, constantPool, true);
    }

    /* JADX WARN: Can't fix incorrect switch cases order, some code will duplicate */
    /* JADX WARN: Failed to find 'out' block for switch in B:6:0x0085. Please report as an issue. */
    /* JADX WARN: Removed duplicated region for block: B:49:0x059f  */
    /* JADX WARN: Removed duplicated region for block: B:51:0x05bb  */
    /*
        Code decompiled incorrectly, please refer to instructions dump.
        To view partially-correct code enable 'Show inconsistent code' option in preferences
    */
    public static java.lang.String codeToString(org.apache.bcel.util.ByteSequence r6, org.apache.bcel.classfile.ConstantPool r7, boolean r8) throws java.io.IOException {
        /*
            Method dump skipped, instructions count: 2273
            To view this dump change 'Code comments level' option to 'DEBUG'
        */
        throw new UnsupportedOperationException("Method not decompiled: org.apache.bcel.classfile.Utility.codeToString(org.apache.bcel.util.ByteSequence, org.apache.bcel.classfile.ConstantPool, boolean):java.lang.String");
    }

    public static String compactClassName(String str) {
        return compactClassName(str, true);
    }

    public static String compactClassName(String str, boolean chopit) {
        return compactClassName(str, "java.lang.", chopit);
    }

    public static String compactClassName(String str, String prefix, boolean chopit) {
        int len = prefix.length();
        String str2 = pathToPackage(str);
        if (chopit && str2.startsWith(prefix) && str2.substring(len).indexOf(46) == -1) {
            str2 = str2.substring(len);
        }
        return str2;
    }

    public static String pathToPackage(String str) {
        return str.replace('/', '.');
    }

    public static String convertString(String label) {
        char[] ch = label.toCharArray();
        StringBuilder buf = new StringBuilder();
        for (char element : ch) {
            switch (element) {
                case '\n':
                    buf.append("\\n");
                    break;
                case '\r':
                    buf.append("\\r");
                    break;
                case '\"':
                    buf.append("\\\"");
                    break;
                case '\'':
                    buf.append("\\'");
                    break;
                case '\\':
                    buf.append("\\\\");
                    break;
                default:
                    buf.append(element);
                    break;
            }
        }
        return buf.toString();
    }

    private static int countBrackets(String brackets) {
        char[] chars = brackets.toCharArray();
        int count = 0;
        boolean open = false;
        for (char c : chars) {
            switch (c) {
                case ElementValue.ARRAY /* 91 */:
                    if (open) {
                        throw new IllegalArgumentException("Illegally nested brackets:" + brackets);
                    }
                    open = true;
                    break;
                case ']':
                    if (!open) {
                        throw new IllegalArgumentException("Illegally nested brackets:" + brackets);
                    }
                    open = false;
                    count++;
                    break;
            }
        }
        if (open) {
            throw new IllegalArgumentException("Illegally nested brackets:" + brackets);
        }
        return count;
    }

    public static byte[] decode(String s, boolean uncompress) throws IOException {
        JavaReader jr = new JavaReader(new CharArrayReader(s.toCharArray()));
        Throwable th = null;
        try {
            ByteArrayOutputStream bos = new ByteArrayOutputStream();
            Throwable th2 = null;
            while (true) {
                try {
                    try {
                        int ch = jr.read();
                        if (ch < 0) {
                            break;
                        }
                        bos.write(ch);
                    } finally {
                    }
                } catch (Throwable th3) {
                    if (bos != null) {
                        if (th2 != null) {
                            try {
                                bos.close();
                            } catch (Throwable th4) {
                                th2.addSuppressed(th4);
                            }
                        } else {
                            bos.close();
                        }
                    }
                    throw th3;
                }
            }
            byte[] bytes = bos.toByteArray();
            if (bos != null) {
                if (0 != 0) {
                    try {
                        bos.close();
                    } catch (Throwable th5) {
                        th2.addSuppressed(th5);
                    }
                } else {
                    bos.close();
                }
            }
            if (uncompress) {
                GZIPInputStream gis = new GZIPInputStream(new ByteArrayInputStream(bytes));
                byte[] tmp = new byte[bytes.length * 3];
                int count = 0;
                while (true) {
                    int b = gis.read();
                    if (b < 0) {
                        break;
                    }
                    int i = count;
                    count++;
                    tmp[i] = (byte) b;
                }
                bytes = Arrays.copyOf(tmp, count);
            }
            return bytes;
        } finally {
            if (jr != null) {
                if (0 != 0) {
                    try {
                        jr.close();
                    } catch (Throwable th6) {
                        th.addSuppressed(th6);
                    }
                } else {
                    jr.close();
                }
            }
        }
    }

    public static String encode(byte[] bytes, boolean compress) throws IOException {
        if (compress) {
            ByteArrayOutputStream baos = new ByteArrayOutputStream();
            Throwable th = null;
            try {
                GZIPOutputStream gos = new GZIPOutputStream(baos);
                Throwable th2 = null;
                try {
                    try {
                        gos.write(bytes, 0, bytes.length);
                        gos.close();
                        bytes = baos.toByteArray();
                        if (gos != null) {
                            if (0 != 0) {
                                try {
                                    gos.close();
                                } catch (Throwable th3) {
                                    th2.addSuppressed(th3);
                                }
                            } else {
                                gos.close();
                            }
                        }
                    } finally {
                    }
                } catch (Throwable th4) {
                    if (gos != null) {
                        if (th2 != null) {
                            try {
                                gos.close();
                            } catch (Throwable th5) {
                                th2.addSuppressed(th5);
                            }
                        } else {
                            gos.close();
                        }
                    }
                    throw th4;
                }
            } finally {
                if (baos != null) {
                    if (0 != 0) {
                        try {
                            baos.close();
                        } catch (Throwable th6) {
                            th.addSuppressed(th6);
                        }
                    } else {
                        baos.close();
                    }
                }
            }
        }
        CharArrayWriter caw = new CharArrayWriter();
        JavaWriter jw = new JavaWriter(caw);
        Throwable th7 = null;
        try {
            try {
                for (byte b : bytes) {
                    int in = b & 255;
                    jw.write(in);
                }
                if (jw != null) {
                    if (0 != 0) {
                        try {
                            jw.close();
                        } catch (Throwable th8) {
                            th7.addSuppressed(th8);
                        }
                    } else {
                        jw.close();
                    }
                }
                return caw.toString();
            } finally {
            }
        } catch (Throwable th9) {
            if (jw != null) {
                if (th7 != null) {
                    try {
                        jw.close();
                    } catch (Throwable th10) {
                        th7.addSuppressed(th10);
                    }
                } else {
                    jw.close();
                }
            }
            throw th9;
        }
    }

    static boolean equals(byte[] a, byte[] b) {
        int size = a.length;
        if (size != b.length) {
            return false;
        }
        for (int i = 0; i < size; i++) {
            if (a[i] != b[i]) {
                return false;
            }
        }
        return true;
    }

    public static String fillup(String str, int length, boolean leftJustify, char fill) {
        int len = length - str.length();
        char[] buf = new char[Math.max(len, 0)];
        Arrays.fill(buf, fill);
        if (leftJustify) {
            return str + new String(buf);
        }
        return new String(buf) + str;
    }

    public static String format(int i, int length, boolean leftJustify, char fill) {
        return fillup(Integer.toString(i), length, leftJustify, fill);
    }

    /* JADX WARN: Failed to find 'out' block for switch in B:5:0x0023. Please report as an issue. */
    public static String getSignature(String type) {
        StringBuilder buf = new StringBuilder();
        char[] chars = type.toCharArray();
        boolean charFound = false;
        boolean delim = false;
        int index = -1;
        int i = 0;
        while (true) {
            if (i < chars.length) {
                switch (chars[i]) {
                    case '\t':
                    case '\n':
                    case '\f':
                    case '\r':
                    case ' ':
                        if (charFound) {
                            delim = true;
                        }
                        i++;
                    case ElementValue.ARRAY /* 91 */:
                        if (!charFound) {
                            throw new IllegalArgumentException("Illegal type: " + type);
                        }
                        index = i;
                        break;
                    default:
                        charFound = true;
                        if (!delim) {
                            buf.append(chars[i]);
                        }
                        i++;
                }
            }
        }
        int brackets = 0;
        if (index > 0) {
            brackets = countBrackets(type.substring(index));
        }
        String type2 = buf.toString();
        buf.setLength(0);
        for (int i2 = 0; i2 < brackets; i2++) {
            buf.append('[');
        }
        boolean found = false;
        for (int i3 = 4; i3 <= 12 && !found; i3++) {
            if (Const.getTypeName(i3).equals(type2)) {
                found = true;
                buf.append(Const.getShortTypeName(i3));
            }
        }
        if (!found) {
            buf.append('L').append(type2.replace('.', '/')).append(';');
        }
        return buf.toString();
    }

    public static boolean isJavaIdentifierPart(char ch) {
        return (ch >= 'a' && ch <= 'z') || (ch >= 'A' && ch <= 'Z') || ((ch >= FREE_CHARS && ch <= '9') || ch == '_');
    }

    public static boolean isSet(int flag, int i) {
        return (flag & pow2(i)) != 0;
    }

    public static String[] methodSignatureArgumentTypes(String signature) throws ClassFormatException {
        return methodSignatureArgumentTypes(signature, true);
    }

    public static String[] methodSignatureArgumentTypes(String signature, boolean chopit) throws ClassFormatException {
        List<String> vec = new ArrayList<>();
        try {
            int index = signature.indexOf(40) + 1;
            if (index <= 0) {
                throw new ClassFormatException("Invalid method signature: " + signature);
            }
            while (signature.charAt(index) != ')') {
                vec.add(typeSignatureToString(signature.substring(index), chopit));
                index += unwrap(CONSUMER_CHARS);
            }
            return (String[]) vec.toArray(ArrayUtils.EMPTY_STRING_ARRAY);
        } catch (StringIndexOutOfBoundsException e) {
            throw new ClassFormatException("Invalid method signature: " + signature, e);
        }
    }

    public static String methodSignatureReturnType(String signature) throws ClassFormatException {
        return methodSignatureReturnType(signature, true);
    }

    public static String methodSignatureReturnType(String signature, boolean chopit) throws ClassFormatException {
        try {
            int index = signature.lastIndexOf(41) + 1;
            if (index <= 0) {
                throw new ClassFormatException("Invalid method signature: " + signature);
            }
            String type = typeSignatureToString(signature.substring(index), chopit);
            return type;
        } catch (StringIndexOutOfBoundsException e) {
            throw new ClassFormatException("Invalid method signature: " + signature, e);
        }
    }

    public static String methodSignatureToString(String signature, String name, String access) {
        return methodSignatureToString(signature, name, access, true);
    }

    public static String methodSignatureToString(String signature, String name, String access, boolean chopit) {
        return methodSignatureToString(signature, name, access, chopit, null);
    }

    public static String methodSignatureToString(String signature, String name, String access, boolean chopit, LocalVariableTable vars) throws ClassFormatException {
        StringBuilder buf = new StringBuilder("(");
        int varIndex = access.contains("static") ? 0 : 1;
        try {
            int index = signature.indexOf(40) + 1;
            if (index <= 0) {
                throw new ClassFormatException("Invalid method signature: " + signature);
            }
            while (signature.charAt(index) != ')') {
                String paramType = typeSignatureToString(signature.substring(index), chopit);
                buf.append(paramType);
                if (vars != null) {
                    LocalVariable l = vars.getLocalVariable(varIndex, 0);
                    if (l != null) {
                        buf.append(" ").append(l.getName());
                    }
                } else {
                    buf.append(" arg").append(varIndex);
                }
                if ("double".equals(paramType) || "long".equals(paramType)) {
                    varIndex += 2;
                } else {
                    varIndex++;
                }
                buf.append(", ");
                index += unwrap(CONSUMER_CHARS);
            }
            String type = typeSignatureToString(signature.substring(index + 1), chopit);
            if (buf.length() > 1) {
                buf.setLength(buf.length() - 2);
            }
            buf.append(")");
            return access + (!access.isEmpty() ? " " : "") + type + " " + name + buf.toString();
        } catch (StringIndexOutOfBoundsException e) {
            throw new ClassFormatException("Invalid method signature: " + signature, e);
        }
    }

    public static String methodTypeToSignature(String ret, String[] argv) throws ClassFormatException {
        StringBuilder buf = new StringBuilder("(");
        if (argv != null) {
            for (String element : argv) {
                String str = getSignature(element);
                if (str.endsWith("V")) {
                    throw new ClassFormatException("Invalid type: " + element);
                }
                buf.append(str);
            }
        }
        buf.append(")").append(getSignature(ret));
        return buf.toString();
    }

    private static int pow2(int n) {
        return 1 << n;
    }

    public static String printArray(Object[] obj) {
        return printArray(obj, true);
    }

    public static String printArray(Object[] obj, boolean braces) {
        return printArray(obj, braces, false);
    }

    public static String printArray(Object[] obj, boolean braces, boolean quote) {
        if (obj == null) {
            return null;
        }
        StringBuilder buf = new StringBuilder();
        if (braces) {
            buf.append('{');
        }
        for (int i = 0; i < obj.length; i++) {
            if (obj[i] != null) {
                buf.append(quote ? "\"" : "").append(obj[i]).append(quote ? "\"" : "");
            } else {
                buf.append("null");
            }
            if (i < obj.length - 1) {
                buf.append(", ");
            }
        }
        if (braces) {
            buf.append('}');
        }
        return buf.toString();
    }

    public static void printArray(PrintStream out, Object[] obj) {
        out.println(printArray(obj, true));
    }

    public static void printArray(PrintWriter out, Object[] obj) {
        out.println(printArray(obj, true));
    }

    public static String replace(String str, String old, String new_) {
        try {
            if (str.contains(old)) {
                StringBuilder buf = new StringBuilder();
                int oldIndex = 0;
                while (true) {
                    int index = str.indexOf(old, oldIndex);
                    if (index == -1) {
                        break;
                    }
                    buf.append((CharSequence) str, oldIndex, index);
                    buf.append(new_);
                    oldIndex = index + old.length();
                }
                buf.append(str.substring(oldIndex));
                str = buf.toString();
            }
        } catch (StringIndexOutOfBoundsException e) {
            System.err.println(e);
        }
        return str;
    }

    public static short searchOpcode(String name) {
        String name2 = name.toLowerCase(Locale.ENGLISH);
        short s = 0;
        while (true) {
            short i = s;
            if (i < Const.OPCODE_NAMES_LENGTH) {
                if (!Const.getOpcodeName(i).equals(name2)) {
                    s = (short) (i + 1);
                } else {
                    return i;
                }
            } else {
                return (short) -1;
            }
        }
    }

    public static int setBit(int flag, int i) {
        return flag | pow2(i);
    }

    public static String signatureToString(String signature) {
        return signatureToString(signature, true);
    }

    public static String signatureToString(String signature, boolean chopit) {
        String typeParams = "";
        int index = 0;
        if (signature.charAt(0) == '<') {
            typeParams = typeParamTypesToString(signature, chopit);
            index = 0 + unwrap(CONSUMER_CHARS);
        }
        if (signature.charAt(index) == '(') {
            String type = typeParams + typeSignaturesToString(signature.substring(index), chopit, ')');
            int index2 = index + unwrap(CONSUMER_CHARS);
            String type2 = type + typeSignatureToString(signature.substring(index2), chopit);
            int unwrap = index2 + unwrap(CONSUMER_CHARS);
            return type2;
        }
        String type3 = typeSignatureToString(signature.substring(index), chopit);
        int index3 = index + unwrap(CONSUMER_CHARS);
        if (typeParams.isEmpty() && index3 == signature.length()) {
            return type3;
        }
        StringBuilder typeClass = new StringBuilder(typeParams);
        typeClass.append(" extends ");
        typeClass.append(type3);
        if (index3 < signature.length()) {
            typeClass.append(" implements ");
            typeClass.append(typeSignatureToString(signature.substring(index3), chopit));
            index3 += unwrap(CONSUMER_CHARS);
        }
        while (index3 < signature.length()) {
            typeClass.append(", ");
            typeClass.append(typeSignatureToString(signature.substring(index3), chopit));
            index3 += unwrap(CONSUMER_CHARS);
        }
        return typeClass.toString();
    }

    public static String toHexString(byte[] bytes) {
        StringBuilder buf = new StringBuilder();
        for (int i = 0; i < bytes.length; i++) {
            short b = byteToShort(bytes[i]);
            String hex = Integer.toHexString(b);
            if (b < 16) {
                buf.append('0');
            }
            buf.append(hex);
            if (i < bytes.length - 1) {
                buf.append(' ');
            }
        }
        return buf.toString();
    }

    public static byte typeOfMethodSignature(String signature) throws ClassFormatException {
        try {
            if (signature.charAt(0) != '(') {
                throw new ClassFormatException("Invalid method signature: " + signature);
            }
            int index = signature.lastIndexOf(41) + 1;
            return typeOfSignature(signature.substring(index));
        } catch (StringIndexOutOfBoundsException e) {
            throw new ClassFormatException("Invalid method signature: " + signature, e);
        }
    }

    public static byte typeOfSignature(String signature) throws ClassFormatException {
        try {
            switch (signature.charAt(0)) {
                case '!':
                case '*':
                case '+':
                    return typeOfSignature(signature.substring(1));
                case '\"':
                case '#':
                case ESCAPE_CHAR /* 36 */:
                case '%':
                case '&':
                case '\'':
                case '(':
                case ')':
                case ',':
                case '-':
                case '.':
                case '/':
                case FREE_CHARS /* 48 */:
                case '1':
                case '2':
                case '3':
                case '4':
                case '5':
                case '6':
                case '7':
                case '8':
                case '9':
                case ':':
                case ';':
                case '<':
                case '=':
                case '>':
                case '?':
                case ElementValue.ANNOTATION /* 64 */:
                case 'A':
                case 'E':
                case 'G':
                case 'H':
                case 'K':
                case 'M':
                case 'N':
                case 'O':
                case 'P':
                case 'Q':
                case 'R':
                case 'U':
                case 'W':
                case 'X':
                case 'Y':
                default:
                    throw new ClassFormatException("Invalid method signature: " + signature);
                case ElementValue.PRIMITIVE_BYTE /* 66 */:
                    return (byte) 8;
                case ElementValue.PRIMITIVE_CHAR /* 67 */:
                    return (byte) 5;
                case ElementValue.PRIMITIVE_DOUBLE /* 68 */:
                    return (byte) 7;
                case ElementValue.PRIMITIVE_FLOAT /* 70 */:
                    return (byte) 6;
                case ElementValue.PRIMITIVE_INT /* 73 */:
                    return (byte) 10;
                case ElementValue.PRIMITIVE_LONG /* 74 */:
                    return (byte) 11;
                case 'L':
                case 'T':
                    return (byte) 14;
                case ElementValue.PRIMITIVE_SHORT /* 83 */:
                    return (byte) 9;
                case 'V':
                    return (byte) 12;
                case ElementValue.PRIMITIVE_BOOLEAN /* 90 */:
                    return (byte) 4;
                case ElementValue.ARRAY /* 91 */:
                    return (byte) 13;
            }
        } catch (StringIndexOutOfBoundsException e) {
            throw new ClassFormatException("Invalid method signature: " + signature, e);
        }
    }

    private static String typeParamTypesToString(String signature, boolean chopit) {
        StringBuilder typeParams = new StringBuilder("<");
        typeParams.append(typeParamTypeToString(signature.substring(1), chopit));
        int i = 1;
        int unwrap = unwrap(CONSUMER_CHARS);
        while (true) {
            int index = i + unwrap;
            if (signature.charAt(index) != '>') {
                typeParams.append(", ");
                typeParams.append(typeParamTypeToString(signature.substring(index), chopit));
                i = index;
                unwrap = unwrap(CONSUMER_CHARS);
            } else {
                wrap(CONSUMER_CHARS, index + 1);
                return typeParams.append(">").toString();
            }
        }
    }

    private static String typeParamTypeToString(String signature, boolean chopit) {
        int index = signature.indexOf(58);
        if (index <= 0) {
            throw new ClassFormatException("Invalid type parameter signature: " + signature);
        }
        StringBuilder typeParam = new StringBuilder(signature.substring(0, index));
        int index2 = index + 1;
        if (signature.charAt(index2) != ':') {
            typeParam.append(" extends ");
            typeParam.append(typeSignatureToString(signature.substring(index2), chopit));
            index2 += unwrap(CONSUMER_CHARS);
        }
        while (signature.charAt(index2) == ':') {
            int index3 = index2 + 1;
            typeParam.append(" & ");
            typeParam.append(typeSignatureToString(signature.substring(index3), chopit));
            index2 = index3 + unwrap(CONSUMER_CHARS);
        }
        wrap(CONSUMER_CHARS, index2);
        return typeParam.toString();
    }

    private static String typeSignaturesToString(String signature, boolean chopit, char term) {
        StringBuilder typeList = new StringBuilder(signature.substring(0, 1));
        int index = 1;
        if (signature.charAt(1) != term) {
            typeList.append(typeSignatureToString(signature.substring(1), chopit));
            index = 1 + unwrap(CONSUMER_CHARS);
        }
        while (signature.charAt(index) != term) {
            typeList.append(", ");
            typeList.append(typeSignatureToString(signature.substring(index), chopit));
            index += unwrap(CONSUMER_CHARS);
        }
        wrap(CONSUMER_CHARS, index + 1);
        return typeList.append(term).toString();
    }

    public static String typeSignatureToString(String signature, boolean chopit) throws ClassFormatException {
        int fromIndex;
        int consumedChars;
        wrap(CONSUMER_CHARS, 1);
        try {
            switch (signature.charAt(0)) {
                case ElementValue.PRIMITIVE_BYTE /* 66 */:
                    return "byte";
                case ElementValue.PRIMITIVE_CHAR /* 67 */:
                    return "char";
                case ElementValue.PRIMITIVE_DOUBLE /* 68 */:
                    return "double";
                case 'E':
                case 'G':
                case 'H':
                case 'K':
                case 'M':
                case 'N':
                case 'O':
                case 'P':
                case 'Q':
                case 'R':
                case 'U':
                case 'W':
                case 'X':
                case 'Y':
                default:
                    throw new ClassFormatException("Invalid signature: `" + signature + "'");
                case ElementValue.PRIMITIVE_FLOAT /* 70 */:
                    return "float";
                case ElementValue.PRIMITIVE_INT /* 73 */:
                    return "int";
                case ElementValue.PRIMITIVE_LONG /* 74 */:
                    return "long";
                case 'L':
                    int fromIndex2 = signature.indexOf(60);
                    if (fromIndex2 < 0) {
                        fromIndex = 0;
                    } else {
                        fromIndex = signature.indexOf(62, fromIndex2);
                        if (fromIndex < 0) {
                            throw new ClassFormatException("Invalid signature: " + signature);
                        }
                    }
                    int index = signature.indexOf(59, fromIndex);
                    if (index < 0) {
                        throw new ClassFormatException("Invalid signature: " + signature);
                    }
                    int bracketIndex = signature.substring(0, index).indexOf(60);
                    if (bracketIndex < 0) {
                        wrap(CONSUMER_CHARS, index + 1);
                        return compactClassName(signature.substring(1, index), chopit);
                    }
                    int fromIndex3 = signature.indexOf(59);
                    if (fromIndex3 < 0) {
                        throw new ClassFormatException("Invalid signature: " + signature);
                    }
                    if (fromIndex3 < bracketIndex) {
                        wrap(CONSUMER_CHARS, fromIndex3 + 1);
                        return compactClassName(signature.substring(1, fromIndex3), chopit);
                    }
                    StringBuilder type = new StringBuilder(compactClassName(signature.substring(1, bracketIndex), chopit)).append("<");
                    int consumedChars2 = bracketIndex + 1;
                    if (signature.charAt(consumedChars2) == '+') {
                        type.append("? extends ");
                        consumedChars2++;
                    } else if (signature.charAt(consumedChars2) == '-') {
                        type.append("? super ");
                        consumedChars2++;
                    }
                    if (signature.charAt(consumedChars2) == '*') {
                        type.append("?");
                        consumedChars = consumedChars2 + 1;
                    } else {
                        type.append(typeSignatureToString(signature.substring(consumedChars2), chopit));
                        consumedChars = unwrap(CONSUMER_CHARS) + consumedChars2;
                        wrap(CONSUMER_CHARS, consumedChars);
                    }
                    while (signature.charAt(consumedChars) != '>') {
                        type.append(", ");
                        if (signature.charAt(consumedChars) == '+') {
                            type.append("? extends ");
                            consumedChars++;
                        } else if (signature.charAt(consumedChars) == '-') {
                            type.append("? super ");
                            consumedChars++;
                        }
                        if (signature.charAt(consumedChars) == '*') {
                            type.append("?");
                            consumedChars++;
                        } else {
                            type.append(typeSignatureToString(signature.substring(consumedChars), chopit));
                            consumedChars = unwrap(CONSUMER_CHARS) + consumedChars;
                            wrap(CONSUMER_CHARS, consumedChars);
                        }
                    }
                    int consumedChars3 = consumedChars + 1;
                    type.append(">");
                    if (signature.charAt(consumedChars3) == '.') {
                        type.append(".");
                        type.append(typeSignatureToString("L" + signature.substring(consumedChars3 + 1), chopit));
                        wrap(CONSUMER_CHARS, unwrap(CONSUMER_CHARS) + consumedChars3);
                        return type.toString();
                    }
                    if (signature.charAt(consumedChars3) != ';') {
                        throw new ClassFormatException("Invalid signature: " + signature);
                    }
                    wrap(CONSUMER_CHARS, consumedChars3 + 1);
                    return type.toString();
                case ElementValue.PRIMITIVE_SHORT /* 83 */:
                    return "short";
                case 'T':
                    int index2 = signature.indexOf(59);
                    if (index2 < 0) {
                        throw new ClassFormatException("Invalid type variable signature: " + signature);
                    }
                    wrap(CONSUMER_CHARS, index2 + 1);
                    return compactClassName(signature.substring(1, index2), chopit);
                case 'V':
                    return "void";
                case ElementValue.PRIMITIVE_BOOLEAN /* 90 */:
                    return "boolean";
                case ElementValue.ARRAY /* 91 */:
                    StringBuilder brackets = new StringBuilder();
                    int n = 0;
                    while (signature.charAt(n) == '[') {
                        brackets.append("[]");
                        n++;
                    }
                    String type2 = typeSignatureToString(signature.substring(n), chopit);
                    int temp = unwrap(CONSUMER_CHARS) + n;
                    wrap(CONSUMER_CHARS, temp);
                    return type2 + brackets.toString();
            }
        } catch (StringIndexOutOfBoundsException e) {
            throw new ClassFormatException("Invalid signature: " + signature, e);
        }
    }

    private static int unwrap(ThreadLocal<Integer> tl) {
        return tl.get().intValue();
    }

    private static void wrap(ThreadLocal<Integer> tl, int value) {
        tl.set(Integer.valueOf(value));
    }
}
